local L = LibStub("AceLocale-3.0"):GetLocale("SliceCommander")

function SliceCommander:UNIT_HEALTH(_, unitID)
	if unitID == "player" then
		SliceCommander:HealthBar('HEALTH')
	elseif unitID == "target" then
		SliceCommander:HealthBar('THEALTH')
	end
end

function SliceCommander:PLAYER_REGEN_DISABLED()
	SliceCommander:HealthBar('HEALTH')
	SliceCommander:HealthBar('THEALTH')
	SliceCommander:TestDP()
	SliceCommander:Energy()
end

--1:unknown 2:sourceGUID 3:sourceName 4:sourceFlags 5:sourceFlags2 6:destGUID 7:destName 8:destFlags 9:destFlags2
--10:spellId 11:spellName 12:spellSchool
--13:extraSpellID 14:extraSpellName	15:extraSchool
function SliceCommander:COMBAT_LOG_EVENT_UNFILTERED()
	local _, event, _, _, sourceName, _, _, _, _, _, _, _, _, _, extraSpellID = CombatLogGetCurrentEventInfo()
	if event == "SPELL_INTERRUPT" and sourceName == UnitName("player") and SC.SET.kickEnable == true then
		local message = SC.SET.kickMessage
		if SC.SET.kickSpell then
			local link = GetSpellLink(extraSpellID)
			message = link..' > '..message
		end
		if SC.SET.kickChannel['INSTANCE_CHAT'] and IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
			SendChatMessage(message, 'INSTANCE_CHAT')
		elseif SC.SET.kickChannel['RAID'] and IsInRaid() then
			SendChatMessage(message, 'RAID')
		elseif SC.SET.kickChannel['PARTY'] and IsInGroup() then
			SendChatMessage(message, 'PARTY')
		elseif SC.SET.kickChannel['SAY'] then
			SendChatMessage(message, 'SAY')
		end
	end
end

function SliceCommander:UNIT_POWER_FREQUENT(_, unit, powerType)
	if unit == "player" and powerType == "COMBO_POINTS" then
		SliceCommander:SetComboPts()
		SliceCommander:ChangeAnchor()
	end
	if unit == "player" and powerType == "ENERGY" then
		SliceCommander:Energy()--energy value and play sound if necessary
	end
end

function SliceCommander:PLAYER_TARGET_CHANGED()
	SliceCommander:HealthBar('THEALTH')
	SliceCommander:ChangeAnchor()
end

function SliceCommander:UNIT_SPELLCAST_START(_, unit, _, spellId)
	if unit == "player" and SC.SET.displayGCD then
		local start, dur = GetSpellCooldown(spellId)
		if dur and dur > 0 and dur <= 1.5 then
			SC.OTHER['AS']['expire'] = start+dur
			SC.OTHER['AS']['obj']:SetMinMaxValues(0, dur)
			SC.OTHER['AS']['obj']:Show()
		end
	end
end

function SliceCommander:UNIT_MAXPOWER(_, UnitId, PowyerType)
	if SC.initDone and UnitId == "player" and PowyerType == "COMBO_POINTS" then
		--Update CP Bar
		local maxCP = UnitPowerMax("player", Enum.PowerType.ComboPoints)

		for i = 1, maxCP do
			SC.BARS['CP']['obj'].combos[i].bg:SetVertexColor(unpack(SC.SET.CP[maxCP][i]))
		end
		for i = 1, 10 do
			SC.BARS['CP']['obj'].combos[i].bg:Hide()
		end

		SliceCommander:SetTimersHeight(SC.SET.TimerHeight)
		SliceCommander:SetWidth(SC.SET.Width)
		SliceCommander:SetComboPts()
	end
end

function SliceCommander:UNKNOWN()
	--DO NOTHING TRY TO AVOID UBKOWN CALL...
end

function SliceCommander:ACTIVE_TALENT_GROUP_CHANGED()
	if SC.initDone then
		SliceCommander:SetSparks()--MOVE SPARK for the current spec

		local aSpec = GetSpecialization()
		SC.OTHER['POISON']['obj'].button:SetAttribute("spell1", SC_SPELL[SC.SET.poison.mh[aSpec]])
		SC.OTHER['POISON']['obj'].button:SetAttribute("spell2", SC_SPELL[SC.SET.poison.oh[aSpec]])
		if SC.SET.poison.oh == 'none' then
			SC.OTHER['POISON']['obj'].button:SetAttribute("spell2", nil)
		end
	end
end

function SliceCommander:AZERITE_ESSENCE_UPDATE()
	SliceCommander:SetEssence()
end

function SliceCommander:PLAYER_LOGIN()
	_G['SC'].preventSound = true
	SliceCommander:TestCurrentVersion()
	_G['SC'].initDone = true
	_G['SC'].order = 0
	_G['SC'].maxPos = 0
	_G['SC'].maxCDPos = 0
	_G['SC'].previouKyrCount = 0
	SliceCommander:TestValueOptions()
	SliceCommander:SetEssence()
	SliceCommander:InitConfigurationTable()
	SliceCommander:InitFont()
	SliceCommander:InitConstantValue()
	SliceCommander:CreateCore()
	SliceCommander:InitBar()
	SliceCommander:InitIcon()
	SliceCommander:InitGCDBar()
	SliceCommander:ApplyConf()
	SliceCommander:SetEssence()
	SliceCommander:ChangeAnchor()
	_G['SC'].preventSound = false
	collectgarbage()
	UpdateAddOnMemoryUsage()
	print('|cffc14f2cSlice|r|cff40a740Commander|r v'..GetAddOnMetadata("SliceCommander", "Version")..' ('..L["memory use"]..':'..floor(GetAddOnMemoryUsage("SliceCommander"))..' kb)')
end

function SliceCommander:OnInitialize()
	if SC == nil then
		_G['SC'] = {}
	end
	_G['SC'].initDone = false
	local _, englishClass = UnitClass("player")
	
	local spec = GetSpecialization()
	if spec == 5 and englishClass == "ROGUE" then --newly created toon
		print(L["You have to choose a spec to use SliceCommander."])
		SliceCommander:OnDisable()
		return 0
	end
	
	if englishClass == "ROGUE" then
		SliceCommander_LoadTukuiApi()
		SliceCommander:LoadPopup()
		SliceCommander:Contruct_StaticPopups()

		--HAS TO BE Pixel Perfect scale
		SliceCommander:UIScale('PLAYER_LOGIN')

		SliceCommander:RegisterEvent('UI_SCALE_CHANGED', 'UIScale')
		SliceCommander:RegisterEvent("PLAYER_REGEN_DISABLED")
		SliceCommander:RegisterEvent("PLAYER_REGEN_ENABLED", "PLAYER_REGEN_DISABLED")
		SliceCommander:RegisterEvent("UNIT_POWER_FREQUENT")
		SliceCommander:RegisterEvent("PLAYER_TARGET_CHANGED")
		SliceCommander:RegisterEvent("UNIT_HEALTH")
		SliceCommander:RegisterEvent("PLAYER_LOGIN")
		SliceCommander:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		SliceCommander:RegisterEvent("UNIT_SPELLCAST_START")
		SliceCommander:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED","UNIT_SPELLCAST_START")
		SliceCommander:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		SliceCommander:RegisterEvent("PLAYER_TALENT_UPDATE", "ACTIVE_TALENT_GROUP_CHANGED")
		SliceCommander:RegisterEvent("UNIT_MAXPOWER")
		SliceCommander:RegisterEvent("AZERITE_ESSENCE_UPDATE")
	else
		SliceCommander:OnDisable()
		return 0
	end
end

function SliceCommander:TestKYRIAN()
	local _, spellId
	local count = 0
	local kyrBuff = {
		[323558] = 2, -- 2nd CP
		[323559] = 3, -- 3rd CP
		[323560] = 4, -- 4th CP
		[354838] = 5, -- 5th CP
	}
	
	for k, v in pairs(kyrBuff) do
		_, _, _, _, _, _, _, _, _, spellId = SliceCommander:UnitAura("player", k, "HELPFUL PLAYER")
		if spellId then
			count = v
		end 
	end
	
	if count ~= _G['SC'].previouKyrCount then
		if _G['SC'].previouKyrCount > 0 then
			_G['SC'].BARS['CP']['obj'].combos[_G['SC'].previouKyrCount].anim:Stop()
		end
		if count > 0 then
			print(count)
			_G['SC'].BARS['CP']['obj'].combos[count].anim:Play()
		end
		_G['SC'].previouKyrCount = count
	end
end

function SliceCommander:OnUpdate()
	SliceCommander:Latency()--timer reference and latency
	SliceCommander:GCD()--GCD
	SliceCommander:TargetThreat()--Threat

	for k, v in pairs(SC.BARS) do
		if SC.BARS[k]['type'] ~= nil then--1=BUFF 2=DEBUFF 3=SELF DEBUF
			if SC.BARS[k]['type'] == 1 or SC.BARS[k]['type'] == 2 or SC.BARS[k]['type'] == 3 then
				SliceCommander:UpdateBar(k)
			elseif SC.BARS[k]['type'] == 4 then--CD
				SliceCommander:AbilityCDBar(k)
			elseif SC.BARS[k]['type'] == 5 then--CHARGE CD
				SliceCommander:ChargeAbilityCDBar(k)
			elseif SC.BARS[k]['type'] == 6 then--MULTIPLE BUFF BAR
				SliceCommander:MultipleAbilityBar(k)
			end
		end
	end

	SliceCommander:TestKYRIAN()--DP Icon
	SliceCommander:TestDP()--DP Icon
end